import requests
import datetime

def getUserInfo(username):
    print("Getting user info ............")
    try:
        req = requests.get(
            "https://codeforces.com/api/user.info?handles=" + username)

        data = req.json()

        if data["status"] == "OK":

            organization = ""

            if "organization" in data["result"][0]:
                organization = data["result"][0]["organization"]
            else:
                organization = "none"

            return {
                "handle": data["result"][0]["handle"],
                "rating": str(data["result"][0]["rating"]),
                "maxRating": str(data["result"][0]["maxRating"]),
                "contribution": str(data["result"][0]["contribution"]),
                "rank": data["result"][0]["rank"],
                "maxRank": data["result"][0]["maxRank"],
                "registrationTimeSeconds": datetime.datetime.utcfromtimestamp(data["result"][0]["registrationTimeSeconds"]).strftime('%Y'),
                "organization": organization,
                "avatar": data["result"][0]["avatar"]

            }

    except Exception as error:
        print(error)


def getSubmissions(username):

    apiUrl = "https://codeforces.com/api/user.status"

    parameters = {"handle": username}

    print("Generating Submissions .......")

    submissions = []

    try:
        req = requests.get(url=apiUrl, params=parameters)

        data = req.json()

        # print(req.json())

        if(data['status'] == 'OK'):
            # submissions = data['result'][0]

            demo = data['result']
            rating = " "
            contest_id = " "

            for i in range(len(demo)):

                if "rating" in demo[i]["problem"]:

                    rating = demo[i]["problem"]["rating"]

                else:
                    rating = "undefined"

                if "contestId" in demo[i]["problem"]:

                    contest_id = str(demo[i]["problem"]["contestId"])

                else:
                    rating = "undefined"

                if(demo[i]["verdict"] == 'OK'):
                    submissions.append({
                        "problem_name": demo[i]["problem"]["name"],

                        "index": demo[i]["problem"]["index"],

                        "submission_id": str(demo[i]["id"]),

                        "contest_id": str(contest_id),

                        "problem_link": "https://codeforces.com/contest/"+contest_id+"/problem/"+demo[i]["problem"]["index"],

                        "solution_link": "https://codeforces.com/contest/"+contest_id+"/submission/" + str(demo[i]["id"]),

                        "rating": str(rating),

                        "tags": ', '.join([str(elem) for elem in (demo[i]["problem"]["tags"])]),

                        "programmingLanguage": demo[i]["programmingLanguage"],


                        "submission_time": datetime.datetime.utcfromtimestamp(
                            demo[i]["creationTimeSeconds"]).strftime('%d %B %Y %H:%M:%S'),
                    })

            unique_submissions = list(
                {v['problem_name']: v for v in submissions}.values())

            return unique_submissions

    except requests.exceptions.RequestException as error:
        print(error)





# Declare Varibale tags

table_start = "<table>"
table_end = "</table>"
table_row_start = "<tr>"
table_row_end = "</tr>"
table_head_start = "<th>"
table_head_end = "</th>"
table_data_start = "<td>"
table_data_end = "</td>"
href = "href="
link_start = "<a "
end = ">"
link_end = "</a>"
new_line = "\n"

def tableDataCreate(data):
    # "<td>data</td>"
    html = table_data_start + data + table_data_end
    # print(html)
    return html


def linkCreate(link, value):
    # "<a href="link" > value </a>"
    html = link_start + href + link + end + value + link_end
    return html

# def tableRow(n,datas):
#     for i in range(datas):




import subprocess as sp 

init = "git init"
add_files = "git add ."
commit = "git commit -m 'auto generated by [cp-tool]'"
branch = "git branch -M main"
push = "git push origin main --force"


def initGit ():

    repo_url = input("Repository url : ")
    add_origin = "git remote add origin "+repo_url

    sp.call(init,shell=True)
    sp.call(add_files,shell=True)
    sp.call(commit,shell=True)
    sp.call(branch,shell=True)
    sp.call(add_origin,shell=True)
    sp.call(push,shell=True)

def updateRepo():

    sp.call(add_files,shell=True)
    sp.call(commit,shell=True)
    sp.call(push,shell=True)




import cpTool.customModules.htmlGenerator as html

def generateMarkdownFile(file,repo_name,user_info,repos):

    print("Creating markdown file .......")

    file.write("# " + repo_name + "\n\n")

    file.write("## Intro\n\n")

    file.write(">\tAutomated by [cp-tool](https://github.com/jspw/cp-tool)\n\n")

    file.write(
        "This is a repository to keep track of my problem solving practice.\nFor now, It contains all the problems I have solved at \n- **[Codeforces](https://codeforces.com/)** \n\n")

    file.write("## User Details ([" + user_info["handle"] +
               "](https://codeforces.com/profile/" + user_info["handle"] + "))\n\n")

    file.write("```.json\n")

    file.write("{\n")

    file.write('\t"handle" : "' + user_info['handle'] + '",\n')
    file.write('\t"organization" : "' + user_info['organization'] + '",\n')
    file.write('\t"rank" : "' + user_info['rank'] + '",\n')
    file.write('\t"rating" : ' + user_info['rating'] + ',\n')
    file.write('\t"contribution" : ' + user_info['contribution'] + ',\n')
    file.write('\t"maxRank" : "' + user_info['maxRank'] + '",\n')
    file.write('\t"maxRating" : ' + user_info['maxRating'] + ',\n')
    file.write('\t"joined" : "' +
               user_info['registrationTimeSeconds'] + '",\n')

    file.write("}\n")

    file.write("\n```\n")

    file.write("## Submissions \n\n")

    file.write(
        '<table align="center" border = "0px" cellpadding ="2px" cellspacing ="2px" >\n')

    file.write("<tr><th>#</th><th>Probelm</th><th>Catagory</th><th>Rating</th><th>Tags</th><th>Solution</th><th>Submission Time</th></tr>\n")

    for i in range(len(repos)):

        table_body = html.table_row_start + html.tableDataCreate(str(len(repos)-i)) + html.tableDataCreate(html.linkCreate(repos[i]["problem_link"], repos[i]["problem_name"])) + html.tableDataCreate(repos[i]["index"]) + html.tableDataCreate(repos[i]["rating"]) + html.tableDataCreate(
            repos[i]["tags"]) + html.tableDataCreate(html.linkCreate(repos[i]["solution_link"], repos[i]["programmingLanguage"])) + html.tableDataCreate(repos[i]["submission_time"]) + html.table_row_end

        file.write(table_body)

    file.write("</table>\n")


def createReadmeFile(file_name,repo_name,user_info,repos):
    file = open(file_name, 'w')

    generateMarkdownFile(file,repo_name,user_info,repos)

    file.close()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cp-tool",
    version="1.1.1",
    author="jspw",
    author_email="mhshifat757@gmail.com",
    description="cp-tool is an auto generator tool for solved problems at different online judges",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jspw/cp-tool",
    license="MIT License",
    packages=setuptools.find_packages(),
    install_requires=["requests"],
    classifiers=[
        "Programming Language :: Python :: 3",
       "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        "console_scripts": [
            "cp-tool=cpTool.__main__:main",
        ]
    },
)



import os
import sys
import subprocess as sp

import cpTool.customModules.readmeFIleGenerator as readme
import cpTool.git as git
import cpTool.api as api


def get_arguments():
    argument = sys.argv

    if len(argument) == 1 or len(argument) > 2:
        print("\n[-] Please use --help  for more info")
        exit()

    else:
        if argument[1] == "--cf" or argument[1] == "--codeforces":
            return "cf"
            # print("Ok cf")

        elif argument[1] == "--init":

            # print("Generating git")
            return "git"

        elif argument[1] == "-h" or argument[1] == "--help":

            print("\nusage: cp-tool [option] ..... \n")

            print(
                "--init \t : \t to initialize git repo\n--cf \t : \t to generate solved problems from codeforces\n--help \t : \t for more info \n")
            exit()

        else:
            print("\n[-] Please use --help  for more info\n")
            exit()


def createDir():
    while (1):
        try:
            createDir()
            break

        except OSError as error:

            print(error)


def main(argument):
    username = input("Enter codeforces handle : ")

    if argument == "git":

        repo_name = input("Enter your repository name : ")
        current_path = os.getcwd()
        repo_path_name = repo_name
        path = os.path.join(current_path, repo_path_name)

        try:
            os.mkdir(path)
        except Exception as err:
            print(err)
            print("Please try again!")
            exit()

        repo_path = os.path.join(current_path, repo_path_name)

        filePath = os.path.join(repo_path, 'README.md')

        user_info = api.getUserInfo(username)

        repos = api.getSubmissions(username)

        readme.createReadmeFile(filePath, repo_name, user_info, repos)

        os.chdir(repo_path)

        git.initGit()

    elif argument == "cf":

        repo_path = current_path = os.getcwd()

        repo_name = os.path.basename(repo_path)

        filePath = os.path.join(repo_path, 'README.md')

        user_info = api.getUserInfo(username)

        repos = api.getSubmissions(username)

        readme.createReadmeFile(filePath, repo_name, user_info, repos)

        git.updateRepo()

    print("Done!")

    print("Thanks for using cp-tool")


if __name__ == "__main__":
    print('''
                         ____ ____     _____           _ 
                        / ___|  _ \   |_   _|__   ___ | |
                       | |   | |_) |____| |/ _ \ / _ \| |
                       | |___|  __/_____| | (_) | (_) | |
                        \____|_|        |_|\___/ \___/|_|
                                                         
    ''')

    argument = get_arguments()

    main(argument)
